# 时间模块
import time


def get_date_of_today(format="%Y-%m-%d"):
    """获取今天日期

    Args:
        format (str, optional): 时间格式. Defaults to "%Y-%m-%d".
    """
    return time.strftime(format)


def timing(f):
    """测试函数执行时间

    用法 @timing，修饰特定方法即可

    Args:
        f ([type]): 方法
    """
    def wrap(*args):
        import time
        time1 = time.time()
        ret = f(*args)
        time2 = time.time()
        print('{:.2f}s'.format(time2 - time1), end=' ')
        return ret
    return wrap


def time2timestamp(time_str: str, time_format=r'%Y-%m-%d %H:%M:%S'):
    """生成时间戳

    Args:
        time_str (str): 时间字符串，如2018-12-12 00:00:00
        time_format (regexp, optional):时间格式. defaults to r'%Y-%m-%d %H:%M:%S'.
    """
    st = time.strptime(time_str, time_format)
    return int(time.mktime(st))


def timestamp2time(timestamp: int, time_format=r'%Y-%m-%d %H:%M:%S'):
    """时间戳转普通时间

    Args:
        timestamp (int): 时间字符串，如1543391171
        time_format (regexp, optional): 指定转换时间的格式，defaults to r'%Y-%m-%d %H:%M:%S'.
    """
    return time.strftime(time_format, time.localtime(timestamp))
